/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.editor;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.editor.IFormPage;

import com.ibm.team.ui.editor.TeamFormPage;


/**
 * A FormEditorPage is the basic abstraction to contribute editor pages via extension point. Clients have to 
 * sub-class this class to define the content and behavior of their editor page. Every page can be registered
 * with the following extension:
 * 
 * {@code
 * <extension point="com.ibm.team.workitem.ide.ui.editorPages">
 *     <editorPage
 *            id="YOUR_PAGE_ID"
 *            title="YOUR_PAGE_TITLE"
 *            class="YOUR_PAGE_CLASS">
 *     </editorPage>
 * </extension>
 * }
 * 
 * Note: This is NOT the officially supported mechanism for contributing editor pages. 
 * See http://jazz.net/library/article/129 for more details. 
 *
 * @see {@link TeamFormPage}
 */
public abstract class FormEditorPage {

	private FormEditor fEditor;
	private String fId;
	private String fTitle;
	private ExposedTeamFormPage fPage;

	/**
	 * Initializes the editor page.
	 * 
	 * @param editor
	 * 			the editor that owns the page, must not be null
	 * @param id
	 * 			String that uniquely identifies the page
	 * @param title
	 * 			page's title shown on the tab in the UI
	 */
	public void init(FormEditor editor, String id, String title) {
		fId= id;
		fEditor= editor;
		fTitle= title;
		fPage= new ExposedTeamFormPage(fEditor, fId, fTitle);
	}

	/**
	 * @return the page's unique identifier
	 */
	public String getId() {
		return fId;
	}
	
	/**
	 * @return the page's title shown in the UI
	 */
	public String getTitle() {
		return fTitle;
	}
	
	/**
	 * @return the editor input that can be processed by the page
	 */
	public IEditorInput getEditorInput() {
		return fEditor.getEditorInput();
	}
	
	/**
	 * @return the editor site that hosts the page's editor
	 */
	public IEditorSite getEditorSite() {
		return fEditor.getEditorSite();
	}
	
	/**
	 * @return the form page that contains the actual content of the editor page
	 * @see {@link IFormPage}
	 */
	public IFormPage getFormPage() {
		return fPage;
	}
	
	/**
	 * Subclasses must implement this method to create content in the form
	 * hosted in this page.
	 * 
	 * @param form
	 *            the form hosted in this page
	 */
	protected abstract void createFormContent(IManagedForm form);
	
	/**
	 * Dispose widgets and resources acquired for the page. The default implementation
	 * invokes <code>dispose()</code> on the contained {@link IFormPage}. Clients may override this method
	 * to dispose additional resources.
	 * 
	 * @see {@link TeamFormPage}
	 */
	public void dispose() {
		fPage.defaultDispose();
	}
	
	/**
	 * Indicate whether the page can be saved. The default implementation invokes 
	 * <code>isDirty()</code> on the contained {@link IFormPage}. Clients may override this method
	 * to control the page's dirty state differently.
	 * 
	 * @return <code>true</code> if the editor page contains unsaved content, and <code>false</code> otherwise
	 * @see {@link TeamFormPage}
	 */
	public boolean isDirty() {
		return fPage.defaultIsDirty();
	}

	/**
	 * The default implementation sets the focus to the last known active part or delegates the call 
	 * to the managed form if no active part is known. Clients may override this method
	 * to control the focus differently.
	 * 
	 * @see {@link TeamFormPage}
	 */
	public void setFocus() {
		fPage.defaultSetFocus();
	}

	/**
	 * Saves the contents of this part. The default implementation delegates the save to the contained 
	 * {@link IFormPage}. Clients may override this method to control the save process differently.
	 * 
	 * @param monitor 
	 * 			the progress monitor
	 * @see org.eclipse.ui.ISaveablePart#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void doSave(IProgressMonitor monitor) {
		fPage.defaultDoSave(monitor);
	}
	
	private class ExposedTeamFormPage extends TeamFormPage {
		public ExposedTeamFormPage(FormEditor editor, String id, String title) {
			super(editor, id, title);
		}
		@Override
		protected void createFormContent(IManagedForm managedForm) {
			FormEditorPage.this.createFormContent(managedForm);
		}
		@Override
		public void dispose() {
			FormEditorPage.this.dispose();
		};
		
		public boolean isDirty() {
			return FormEditorPage.this.isDirty();
		};
		
		public void doSave(IProgressMonitor monitor) {
			FormEditorPage.this.doSave(monitor);
		};
		
		public void setFocus() {
			FormEditorPage.this.setFocus();
		};
		
		private void defaultDispose() {
			super.dispose();
		};
		private boolean defaultIsDirty() {
			return super.isDirty();
		}
		private void defaultDoSave(IProgressMonitor monitor) {
			super.doSave(monitor);
		}
		private void defaultSetFocus() {
			super.setFocus();
		}
	}
}
