/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.editor.FormEditor;

import com.ibm.team.jface.util.UIUpdaterJob;
import com.ibm.team.links.common.IReference;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.workitem.client.IDetailedStatus;
import com.ibm.team.workitem.client.IWorkItemClient;
import com.ibm.team.workitem.common.internal.util.ItemHandleAwareHashMap;
import com.ibm.team.workitem.common.model.IWorkItemHandle;
import com.ibm.team.workitem.ide.ui.internal.ImagePool;
import com.ibm.team.workitem.ide.ui.internal.WorkItemIDEUIPlugin;
import com.ibm.team.workitem.ide.ui.internal.editor.WorkItemEditorInput;
import com.ibm.team.workitem.rcp.core.WorkItemEditorHistoryEntry;

public class DeleteWorkItemAction extends Action {
	
	private final IWorkbench fWorkbench;
	private IStructuredSelection fSelection;
	
	public DeleteWorkItemAction() {
		fWorkbench= WorkItemIDEUIPlugin.getDefault().getWorkbench();
		setText(Messages.DeleteWorkItemAction_NAME);
		setToolTipText(Messages.DeleteWorkItemAction_TOOLTIP);
		setImageDescriptor(ImagePool.DELETE_WORKITEM_ACTION_ICON);
	}
	
	@Override
	public void run() {
		final List<IWorkItemHandle> workItems= getWorkItems(fSelection);
		if (workItems.isEmpty()) {
			MessageDialog.openWarning(fWorkbench.getActiveWorkbenchWindow().getShell(), 
										Messages.DeleteWorkItemAction_WARN_BOX_TITLE,
										Messages.DeleteWorkItemAction_WARN_BOX_MESSAGE);
		} else {
			boolean confirmDelete= MessageDialog.openConfirm(fWorkbench.getActiveWorkbenchWindow().getShell(), 
					Messages.DeleteWorkItemAction_CONFIRM_BOX_TITLE,
					Messages.DeleteWorkItemAction_CONFIRM_BOX_MESSAGE);
			if (confirmDelete) {
				final ItemHandleAwareHashMap<IWorkItemHandle, IEditorReference> openEditors= getOpenWorkItems();
				final IWorkItemClient client= getWorkItemClient(workItems);
				
				new UIUpdaterJob(Messages.DeleteWorkItemAction_JOB_PROGRESS_MESSAGE) {
					@Override
					public IStatus runInBackground(IProgressMonitor monitor) {
						try {
							return mergeStatus(client.deleteWorkItems(workItems, monitor));
						} catch (TeamRepositoryException ex) {
							return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, ex.getLocalizedMessage());
						}
					}
					public IStatus runInUI(IProgressMonitor monitor) {
						closeOpenEditors(workItems, openEditors);
						return Status.OK_STATUS;
					}
				}.schedule();
			}
		}
	}

	public void setSelection(ISelection selection) {
		List<IWorkItemHandle> workItems= getWorkItems(selection);
		if (workItems.isEmpty()) {
			setEnabled(false);
		} else {
			fSelection= (IStructuredSelection)selection;
			setEnabled(true);
		}
	}

	private void closeOpenEditors(List<IWorkItemHandle> workItems, ItemHandleAwareHashMap<IWorkItemHandle, IEditorReference> openEditors) {
		for (IWorkItemHandle cur : workItems) {
			if (openEditors.containsKey(cur)) {
				IEditorReference curEditor= openEditors.get(cur);
				IEditorPart part= curEditor.getEditor(false);
				if (part instanceof FormEditor) {
					((FormEditor)part).close(false);
				}
			}
		}
	};

	private IWorkItemClient getWorkItemClient(List<IWorkItemHandle> workItems) {
		ITeamRepository repo= (ITeamRepository)workItems.get(0).getOrigin();
		return (IWorkItemClient)repo.getClientLibrary(IWorkItemClient.class);
	}

	private ItemHandleAwareHashMap<IWorkItemHandle, IEditorReference> getOpenWorkItems() {
		ItemHandleAwareHashMap<IWorkItemHandle, IEditorReference> result= new ItemHandleAwareHashMap<IWorkItemHandle, IEditorReference>();
		for (IWorkbenchWindow curWindow : fWorkbench.getWorkbenchWindows()) {
			for (IWorkbenchPage curPage : curWindow.getPages()) {
				IEditorReference[] editors= curPage.getEditorReferences();
				for (IEditorReference curEditor : editors) {
					try {
						IEditorInput input= curEditor.getEditorInput();
						if (input instanceof WorkItemEditorInput) {
							result.put(((WorkItemEditorInput)input).getWorkItem(), curEditor);
						}
					} catch (PartInitException ex) {
						continue;
					}
				}
			}
		}
		return result;
	}
	
	private IStatus mergeStatus(List<IDetailedStatus> status) {
		String message= null;
		Throwable exception= null;
		for (int idx= 0, len= status.size(); idx < len; idx++) {
			IStatus cur= status.get(idx);
			if (!cur.isOK()) {
				if (message== null) message= cur.getMessage();
				if (exception== null) exception= cur.getException();
			}
		}
		return message != null || exception != null
				? new MultiStatus(WorkItemIDEUIPlugin.PLUGIN_ID, -1, status.toArray( new IStatus[status.size()] ), message, exception)
				: Status.OK_STATUS;
	}

	private List<IWorkItemHandle> getWorkItems(ISelection selection) {
		List<IWorkItemHandle> result= new ArrayList<IWorkItemHandle>();
		if (selection == null || selection.isEmpty() || !(selection instanceof StructuredSelection)) {
			return result;
		}

		StructuredSelection sel= (StructuredSelection)selection;
		for (Iterator<?> iterator= sel.iterator(); iterator.hasNext();) {
			Object cur= iterator.next();
			if (cur instanceof WorkItemEditorHistoryEntry) {
				cur= ((WorkItemEditorHistoryEntry)cur).getWorkItem();
			} else if (cur instanceof IReference) {
				IReference curRef= (IReference)cur;
				if (curRef.isItemReference()) {
					cur= curRef.resolve();
				}
			}
			if (cur instanceof IWorkItemHandle && !result.contains(cur)) {
				result.add((IWorkItemHandle)cur);
			}
		}
		return result;
	}
}