/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.actions;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPart;

import com.ibm.team.foundation.common.text.XMLString;
import com.ibm.team.foundation.ide.ui.internal.styledtextviewer.actions.SourceViewerPane;
import com.ibm.team.foundation.ide.ui.internal.styledtextviewer.actions.StyledEditingAction;
import com.ibm.team.foundation.rcp.core.text.IStyledDocument;
import com.ibm.team.repository.common.IContributorHandle;
import com.ibm.team.workitem.client.WorkItemWorkingCopy;
import com.ibm.team.workitem.common.internal.util.ItemHandleAwareHashSet;
import com.ibm.team.workitem.common.model.IComment;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.WorkItemEndPoints;
import com.ibm.team.workitem.ide.ui.internal.ImagePool;
import com.ibm.team.workitem.ide.ui.internal.WorkItemIDEUIPlugin;
import com.ibm.team.workitem.ide.ui.internal.editor.CommentsDocument;
import com.ibm.team.workitem.ide.ui.internal.editor.ExtractWorkItemOperation;
import com.ibm.team.workitem.ide.ui.internal.editor.Util;
import com.ibm.team.workitem.ide.ui.internal.editor.WorkItemEditorInput;
import com.ibm.team.workitem.ide.ui.internal.wizards.ExtractWorkItemWizard;


public class ExtractWorkItemAction extends StyledEditingAction {
	
	private static final int SUMMARY_SIZE= 100;
	
	private static final String[] COPY_ATTRIBUTES= { IWorkItem.CATEGORY_PROPERTY, IWorkItem.FOUND_IN_PROPERTY };
	
	public ExtractWorkItemAction() {
		this(null, null);
	}
	
	public ExtractWorkItemAction(IWorkbenchPart part) {
		this(null, ImagePool.CREATE_DUPLICATE, part, null);
	}
	
	public ExtractWorkItemAction(IWorkbenchPart part, ISourceViewer sourceViewer) {
		this(Messages.ExtractWorkItemAction_EXTRACT_WORKITEM, null, part, sourceViewer);
	}
	
	private ExtractWorkItemAction(String text, ImageDescriptor image, IWorkbenchPart part, ISourceViewer sourceViewer) {
		super(text, "com.ibm.team.workitem.ide.ui.edit.extractworkitem", image, sourceViewer); //$NON-NLS-1$
		setToolTipText(Messages.ExtractWorkItemAction_EXTRACT_FROM_TEXT);
		setDisabledImageDescriptor(ImageDescriptor.createWithFlags(image, SWT.IMAGE_DISABLE));
		setPart(part);
		update();
	}

	public void run() {
		
		update();
		
		if (!isEnabled())
			return;
		
		IEditorPart editorPart= (IEditorPart) getPart();
		WorkItemEditorInput workItemEditorInput= (WorkItemEditorInput) editorPart.getEditorInput();
		WorkItemWorkingCopy sourceWorkingCopy= workItemEditorInput.getWorkingCopy();
		IWorkItem sourceItem= sourceWorkingCopy.getWorkItem();
		
		boolean ok= saveNewWorkItem(editorPart, sourceWorkingCopy);
		if (!ok)
			return;
		
		ISourceViewer sourceViewer= getSourceViewer();
		if (sourceViewer == null) { // handles case in editor header toolbar
			SourceViewerPane sourceViewerPane= (SourceViewerPane) editorPart.getAdapter(SourceViewerPane.class);
			if (sourceViewerPane != null)
				sourceViewer= sourceViewerPane.getActiveSourceViewer();
		}
		if (sourceViewer == null) {
			ExtractWorkItemOperation operation= new ExtractWorkItemOperation(sourceItem, Messages.ExtractWorkItemAction_CREATED, XMLString.EMPTY, XMLString.EMPTY,
					Collections.<IContributorHandle>emptyList(), WorkItemEndPoints.RELATED_WORK_ITEM, COPY_ATTRIBUTES);
			openWizard(operation, sourceItem);
			return;
		}
		
		ITextSelection selection= (ITextSelection) sourceViewer.getSelectionProvider().getSelection();
		XMLString description= XMLString.createFromPlainText(selection.getText());
		
		IDocument document= sourceViewer.getDocument();
		if (document instanceof IStyledDocument) {
			try {
				description= ((IStyledDocument) document).getHTML(selection.getOffset(), selection.getLength());
			} catch (BadLocationException x) {
				WorkItemIDEUIPlugin.getDefault().log(Messages.ExtractWorkItemAction_EXCEPTION_EXTRACTING_WORKITEM, x);
			}
		}
		
		String summary= description.getPlainText();
		summary= summary.replace(System.getProperty("line.separator", "\n"), " ").trim(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		if (summary.length() >= SUMMARY_SIZE) {
			int endPos= summary.indexOf(" ", SUMMARY_SIZE); //$NON-NLS-1$
			if (endPos < 0)
				endPos= SUMMARY_SIZE;
			summary= summary.substring(0, endPos) + " ..."; //$NON-NLS-1$
		}
		
		Set<IContributorHandle> subscriptions= new ItemHandleAwareHashSet<IContributorHandle>();
		if (document instanceof CommentsDocument) {
			
			List<IComment> comments= ((CommentsDocument) document).getComments(selection.getOffset(), selection.getLength());
			for (IComment comment: comments)
				subscriptions.add(comment.getCreator());
			
			subscriptions.remove(sourceWorkingCopy.getTeamRepository().loggedInContributor());
			
		}
		
		ExtractWorkItemOperation operation= new ExtractWorkItemOperation(sourceItem, !selection.isEmpty() ? Messages.ExtractWorkItemAction_EXTRACTED : Messages.ExtractWorkItemAction_CREATED,
				XMLString.createFromPlainText(summary), description, new ArrayList<IContributorHandle>(subscriptions),
				WorkItemEndPoints.RELATED_WORK_ITEM, COPY_ATTRIBUTES);
		openWizard(operation, sourceItem);
	}
	
	private void openWizard(ExtractWorkItemOperation operation, IWorkItem sourceItem) {
		
		IWorkbenchPart part= getPart();
		if (part == null)
			return;
		
		Shell shell= part.getSite().getShell();
		if (shell == null || shell.isDisposed())
			return;
		
		ExtractWorkItemWizard wizard= new ExtractWorkItemWizard(sourceItem.getProjectArea(), operation);
		WizardDialog dialog= new WizardDialog(shell, wizard);
		dialog.open();
	}
	
	public void update() {
		setEnabled(shouldEnable());
		// TODO: setDisabledImageDescriptor(...) didn't work for me.
		setImageDescriptor(isEnabled() ? ImagePool.CREATE_DUPLICATE : ImagePool.CREATE_DUPLICATE_DISABLED);
	}

	private boolean shouldEnable() {
		
		IWorkbenchPart part= getPart();
		if (!(part instanceof IEditorPart))
			return false;
		
		IEditorInput input= ((IEditorPart) part).getEditorInput();
		if (!(input instanceof WorkItemEditorInput && ((WorkItemEditorInput) input).isResolved()))
			return false;
		
		return true;
	}
	
	/**
	 * @return <code>true</code> if the work item is not / no longer new
	 */
	private boolean saveNewWorkItem(final IEditorPart editorPart, WorkItemWorkingCopy workingCopy) {
		
		if (!workingCopy.getWorkItem().isNewItem())
			return true;
		
		Shell shell= editorPart.getSite().getShell();
		if (shell == null || shell.isDisposed())
			return false;
		
		boolean confirmed= MessageDialog.openConfirm(shell, Messages.ExtractWorkItemAction_SAVE_WORKITEM, Messages.ExtractWorkItemAction_NEED_TO_SAVE);
		if (!confirmed)
			return false;
		
		return Util.saveNewWorkItem(editorPart, workingCopy);
	}
}
