/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.resource.LocalResourceManager;
import org.eclipse.jface.resource.ResourceManager;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.events.HyperlinkAdapter;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Hyperlink;

import com.ibm.team.foundation.common.util.IMemento;
import com.ibm.team.process.client.workingcopies.IProcessContainerWorkingCopy;
import com.ibm.team.process.common.ModelElement;
import com.ibm.team.process.common.ModelElements;
import com.ibm.team.process.ide.ui.AbstractConfigurationDataAspectEditor;
import com.ibm.team.process.ide.ui.IProcessAspectEditorSite;
import com.ibm.team.process.ide.ui.ProcessAspect;
import com.ibm.team.workitem.ide.ui.internal.HelpContextIds;
import com.ibm.team.workitem.ide.ui.internal.editor.DecoratedCombo;



public abstract class AbstractWorkItemAspectEditor extends AbstractConfigurationDataAspectEditor implements IDirtyStateTracker {
	private static final String NO_ELEMENTS=Messages.AbstractWorkItemAspectEditor_NO_ELEMENTS;
	private List<ModeledElement> fElements;
	private Composite fContainer;
	private DecoratedCombo fCombo;
	private String fConfigurationDataId;
	
	private ResourceManager fResourceManager= new LocalResourceManager(JFaceResources.getResources());
	
	private String fHelpContextId= null;
	private Button fDuplicateButton;
	private Button fRemoveButton;
	private Hyperlink fValidate;
	
	public AbstractWorkItemAspectEditor(String dataId) {
		fConfigurationDataId= dataId;
	}

	@Override
	public final void dispose() {
		doDispose();
		if (fResourceManager != null) {
			fResourceManager.dispose();
			fResourceManager= null;
		}
	}

	@Override
	public void init(IProcessAspectEditorSite site, ProcessAspect inputAspect) {
		super.init(site, inputAspect);
		init();
	}
	
	private void init() {
		fElements= new ArrayList<ModeledElement>(readElements(getAspect().getConfigurationElement()));
	}
	
	@Override
	public void createControl(Composite parent, FormToolkit toolkit) {
		fContainer= new Composite(parent, SWT.NONE);
		parent.setLayout(new GridLayout(1, false));
		fContainer.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		GridLayout layout = new GridLayout(1, false);
		layout.marginWidth= 0;
		layout.marginHeight= 0;
		layout.verticalSpacing= 0;
		fContainer.setLayout(layout);
		
		Label l= toolkit.createLabel(fContainer, getMessageProvider().getChooseMessage());
		l.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, false));
		Composite elements= toolkit.createComposite(fContainer);
		elements.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		int nOfCol= 1;
		if (canAdd()) {
			nOfCol++;
		}
		if (canDuplicate()) {
			nOfCol++;
		}
		if (canRemove()) {
			nOfCol++;
		}
		if (canValidate()) {
			nOfCol++;
		}
		layout= new GridLayout(nOfCol, false);
		layout.marginWidth= 0;
		layout.marginBottom= 5;
		elements.setLayout(layout);
		fCombo= new DecoratedCombo(elements, SWT.READ_ONLY, DecoratedCombo.NO_DECORATION_OPTION);
		toolkit.adapt(fCombo.getCombo(), true, false);
		fCombo.getLayoutControl().setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, false));
		fCombo.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object element) {
				if (element instanceof String) {
					return (String)element;
				}
				if (element instanceof ModeledElement) {
					String name= ((ModeledElement)element).getName();
					if (name == null || "".equals(name.trim())) { //$NON-NLS-1$
						name= ((ModeledElement)element).getIdentifier();
					}
					return name;
				}
				return super.getText(element);
			}
		});
		fCombo.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				Object o= fCombo.getValue();
				if (o instanceof ModeledElement) {
					inputChanged((ModeledElement)o);
					fCombo.getCombo().setToolTipText(((ModeledElement) o).getIdentifier());
				} else {
					inputChanged(null);
					fCombo.getCombo().setToolTipText(null);
				}
			}
		});
		if (canAdd()) {
			Button createNew= toolkit.createButton(elements, Messages.AbstractWorkItemAspectEditor_ADD, SWT.FLAT);
			createNew.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					ModeledElement element= createNewElement();
					if (element != null) {
						fElements.add(element);
						fCombo.setValueSet(fElements.toArray());
						fCombo.setValue(element);
						if (fElements.size() == 1) {
							fCombo.getCombo().setEnabled(true);
						}
						updateButtonEnablement();
						fCombo.getLayoutControl().getParent().layout();
						setDirty();
					}
				}
			});
			createNew.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, false));
		}
		if (canDuplicate()) {
			fDuplicateButton = toolkit.createButton(elements, Messages.AbstractWorkItemAspectEditor_DUPLICATE, SWT.FLAT);
			fDuplicateButton.setEnabled(false);
			fDuplicateButton.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					ModeledElement element= duplicateElement((ModeledElement)fCombo.getValue());
					if (element != null) {
						fElements.add(element);
						fCombo.setValueSet(fElements.toArray());
						fCombo.setValue(element);
						updateButtonEnablement();
						fCombo.getLayoutControl().getParent().layout();
						setDirty();
					}
				}
			});
			fDuplicateButton.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, false));
		}
		if (canRemove()) {
			fRemoveButton = toolkit.createButton(elements, Messages.AbstractWorkItemAspectEditor_REMOVE, SWT.FLAT);
			fRemoveButton.setEnabled(false);
			fRemoveButton.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					ModeledElement element = (ModeledElement)fCombo.getValue();
					if (removeElement(element)) {
						int index= fElements.indexOf(element)-1;
						if (index < 0) {
							index= 0;
						}
						fElements.remove(element);
						fCombo.setValueSet(fElements.toArray());
						if (fElements.size() > index) {
							fCombo.setValue(fElements.get(index));
						} else if (fElements.size() > 0) {
							fCombo.setValue(fElements.get(0));
						} else {
							fCombo.setValue(NO_ELEMENTS);
							fCombo.getCombo().setEnabled(false);
						}
						updateButtonEnablement();
 						fCombo.getLayoutControl().getParent().layout();
						setDirty();
					}
				}
			});
			fRemoveButton.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, false));
		}
		if (canValidate()) {
			fValidate= toolkit.createHyperlink(elements, getValidateLinkLabel(), SWT.NONE);
			fValidate.addHyperlinkListener(new HyperlinkAdapter() {
				@Override
				public void linkActivated(HyperlinkEvent e) {
					ModeledElement element = (ModeledElement)fCombo.getValue();
					validate(element);
				};
			});
			fValidate.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false));
			
		}
		Composite editorContent= toolkit.createComposite(fContainer);
		editorContent.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		createEditorArea(editorContent, toolkit);
		
		updateCombo();
		updateButtonEnablement();
		if (fHelpContextId != null)
			HelpContextIds.hookHelpListener(parent, fHelpContextId);
		
	}
	
	private void updateCombo() {
		fCombo.setValueSet(fElements.toArray());
		if (fElements == null || fElements.isEmpty()) {
			fCombo.setValue(NO_ELEMENTS);
			fCombo.getCombo().setEnabled(false);
		} else {
			Object selected= fCombo.getValue();
			boolean set= false;
			if (selected instanceof ModeledElement) {
				for (ModeledElement el: fElements) {
					if (el.getIdentifier().equals(((ModeledElement)selected).getIdentifier())) {
						fCombo.setValue(el);
						set= true;
						break;
					}
				}
			}
			if (!set) {
				fCombo.setValue(fElements.get(0));
			}
			fCombo.getCombo().setEnabled(true);
		}
		fCombo.getLayoutControl().getParent().layout();
	}
	
	@Override
	public void revert() {
		init();
		updateCombo();
		setDirty(false);
	}
	
	private void updateButtonEnablement() {
		if (canDuplicate() && fDuplicateButton != null && !fDuplicateButton.isDisposed()) {
			Object[] valueSet= fCombo.getValueSet();
			fDuplicateButton.setEnabled(valueSet.length > 0 && !(valueSet[0] instanceof String));
		}
		if (canRemove() && fRemoveButton != null && !fRemoveButton.isDisposed()) {
			Object[] valueSet= fCombo.getValueSet();
			fRemoveButton.setEnabled(valueSet.length > 0 && !(valueSet[0] instanceof String));
		}
	}
	
	public IProcessContainerWorkingCopy getProcessContainerWorkingCopy() {
		return getAspect().getProcessContainerWorkingCopy();
	}
	
	protected ResourceManager getResourceManager() {
		return fResourceManager;
	}
	
	/**
	 * Should only be used read-only.
	 */
	protected List<? extends ModeledElement> getAllElements() {
		return fElements;
	}
	
	@Override
	public void setDirty() {
		super.setDirty();
	}
	
	protected void setHelpContextId(String helpContextId) {
		fHelpContextId= helpContextId;
	}
	
	protected void updateComboLabels() {
		if (fElements != null && !fCombo.getCombo().isDisposed()) {
			fCombo.setValueSet(fElements.toArray());
		}
	}
	
	public void setComboInput(ModeledElement ele){
		if(fCombo != null){
			fCombo.setValue(ele);
		}
	}
	
	/**
	 * @return Whether the implementing aspect editor can duplicate ModelElements
	 */
	protected boolean canDuplicate() {
		return false;
	}
	
	/**
	 * Duplicate the given element and return the duplicate
	 * This can be a shallow copy if the content can be shared, otherwise
	 * a deep copy should be returned.
	 * @return The duplicate or null in case of user cancel
	 */
	protected ModeledElement duplicateElement(ModeledElement original) {
		return null;
	}
	
	/**
	 * @return Whether the implementing aspect editor can remove ModelElements
	 */
	protected boolean canRemove() {
		return false;
	}
	
	/**
	 * @return Whether the implementing aspect editor can add ModelElements. Defaults to <code>true</code>
	 */
	protected boolean canAdd() {
		return true;
	}
	
	/**
	 * @return Whether the implementing aspect editor can validate model elements. Defaults to <code>false</code>
	 */
	protected boolean canValidate() {
		return false;
	}
	
	/**
	 * Remove the given element. This method should remove the element from the internal model of
	 * the aspect editor an clean up any dependencies. If the return value is <code>true</code>, the 
	 * ModelElement will then removed from the combo.
	 * It is expected that the implementing aspect editor takes care of asking whether the user really
	 * wants to remove.
	 * @return Whether the element was removed
	 */
	protected boolean removeElement(ModeledElement element) {
		return false;
	}
	
	/**
	 * The configuration element may be <code>null</code> (nothing configured yet).
	 */
	protected abstract List<? extends ModeledElement> readElements(ModelElement configurationElement);
	
	/**
	 * @param newElement may be null if no elements exist at all
	 */
	protected abstract void inputChanged(ModeledElement newElement);
	
	protected abstract void createEditorArea(Composite parent, FormToolkit toolkit);
	
	protected abstract void doDispose();
	
	protected abstract ModeledElement createNewElement();
	
	protected abstract IAspectMessageProvider getMessageProvider();
	
	protected String getValidateLinkLabel() {
		return ""; //$NON-NLS-1$
	}

	protected void validate(ModeledElement element) {
		
	}
	@Override
	public final boolean saveState(IMemento memento) {
		memento.putString(ModelElements.CONFIGURATION_DATA_ID_ATTRIBUTE, fConfigurationDataId);
		String namespace = getAspect().getSchemaNamespaceURI();
		if (namespace != null) {
			memento.putString("xmlns", namespace); //$NON-NLS-1$
		}
		commit(fElements, memento);
		return true;  // TODO: should probably return false to delete the configuration element if fElements is empty
	}
	
	/**
	 * @param elements The complete list of elements
	 * @param memento The memento to commit the state into
	 */
	protected abstract void commit(List<? extends ModeledElement> elements, IMemento memento);
	
	@Override
	public boolean needsApplyAndRevertButtons() {
		return false;
	}
	
	public static interface IAspectMessageProvider {
		String getChooseMessage();
		String getSameIdMessage();
		String getEnterIdMessage();

	}
}
