/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.widgets.FormToolkit;

import com.ibm.team.ui.editor.TeamFormPart;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.type.TypeCategory;

/**
 * AddEditRemoveButtonsAndContextMenu is used to create an "Add" button, an
 * "Edit" and a "Remove" buttons on a {@link TeamFormPart} next to a (@link
 * {@link TableViewer}. The class requires that when it is created a class which
 * implements the {@link AddEditRemoveUi} interface is supplied. This is so the
 * appropriate add/edit/remove functions may be called.
 * 
 * The interface will look something like this depiction below, an outer
 * container with a table and on the right hand side of the table are the three
 * buttons that are spaced below the row of column names.
 * 
 * |---------------------------------------------------|
 * | |---------------------------------|               |
 * | |Columns names                    | blank space   |
 * | |Row of data or empty             | Add Button    |
 * | |Row of data or empty             | Edit Button   |
 * | |Row of data or empty             | Remove Button |
 * | |---------------------------------|               |
 * |---------------------------------------------------| 
 * 
 * The add button and add context menu option should always be available unless
 * there is no {#link {@link TypeCategory}.
 * 
 * The edit button and edit context menu option will only be availble when a single
 * item in the table is selected.
 * 
 * The remove button and remove context menu option will be available any time there
 * is one or more items selected in the table.
 * 
 * @author jsteele
 */
public class AddEditRemoveButtonsAndContextMenu 
{	
	private Button addButton;
	/** Possibly null if this grouping doesn't support edit, e.g., WI types are edited inline, don't need an explicit edit. */
	private Button editButton;
	private Button removeButton;
	private AddEditRemoveUi addEditRemoveUi;
	private final boolean fBelowTable;
	private final boolean fShowEdit;
	private boolean addEnabled = true;
	private boolean editEnabled = true;
	private boolean removeEnabled = true;
	
	
	/**
	 * Construct a new AddEditRemoveButtonsAndContextMenu passing in the 
	 * class which contains the real add, edit and delete functions to call.
	 */
	public AddEditRemoveButtonsAndContextMenu(AddEditRemoveUi addEditRemoveUi)
	{
		this(addEditRemoveUi, false, true);
	}
	
	public AddEditRemoveButtonsAndContextMenu(AddEditRemoveUi addEditRemoveUi, boolean belowTable, boolean showEdit)
	{
		this.addEditRemoveUi = addEditRemoveUi;
		fBelowTable = belowTable;
		fShowEdit = showEdit;
	}
	
	
	/**
	 * If the add button is created and has not been disposed of then 
	 * if the TypeCategory is non-null the add button and the table are
	 * enabled, otherwise they are disabled.
	 * 
	 * @param tableViewer the table to be enabled/disabled.
	 * @param enable <code>true</code> if the add button should be enabled.
	 */
	public void updateAddButtonAndTableEnablement(TableViewer tableViewer, boolean enable) 
	{
		if (addButton != null && !addButton.isDisposed()) 
		{
			addButton.setEnabled(enable);
			tableViewer.getControl().setEnabled(enable);
		}
		addEnabled = enable;
	}
	
	
	/**
	 * Constructs the three buttons (add/edit/delete) and places them to the right of 
	 * a {@link TableViewer}.
	 * 
	 * @param teamFormPart the teamFormPart used to extract the toolkit.
	 * @param composite the container to add the buttons to.
	 * @param tableViewer the table the buttons will be next to.
	 */
	public void createButtonsAndMenu(TeamFormPart teamFormPart, Composite composite, final TableViewer tableViewer)
	{
		createButtons(teamFormPart, composite, tableViewer);
		createContextMenu(tableViewer);
	}
	
	
	/**
	 * Override the label for the add button, edit button and remove button. If
	 * the override label is null or the empty string then the original label
	 * will remain.
	 * 
	 * @param addButtonLabel
	 *            The new label for the add button or null to leave unchanged.
	 * @param editButtonLabel
	 *            The new label for the edit button or null to leave unchanged.
	 * @param removeButtonLabel
	 *            The new label for the remove button or null to leave
	 *            unchanged.
	 */
	public void overrideButtonLabels(String addButtonLabel, String editButtonLabel, String removeButtonLabel)
	{
		if (addButtonLabel != null && addButtonLabel.length() != 0) {
			addButton.setText(addButtonLabel);
		}
		
		if (fShowEdit) {
			if (editButtonLabel != null && editButtonLabel.length() != 0) {
				editButton.setText(editButtonLabel);
			}
		}
		
		if (removeButtonLabel != null && removeButtonLabel.length() != 0) {
			removeButton.setText(removeButtonLabel);
		}
	}
	
	
	/**
	 * Allows for overriding the enablement of the add, edit and remove buttons.
	 * In some circumstances a developer may want the buttons to be enabled, for
	 * example don't allow editing of an item if that item is system item, the
	 * same can be said for editing.
	 * 
	 * @param addButtonEnabled
	 *            True if the add button should be enabled, false to disable.
	 * @param editButtonEnabled
	 *            True if the edit button should be enabled, false to disable.
	 * @param removeButtonEnabled
	 *            True if the remove button should be enabled, false to disable.
	 */
	public void updateButtonEnablement(boolean addButtonEnabled, boolean editButtonEnabled, boolean removeButtonEnabled)
	{
		addEnabled = addButtonEnabled;
		editEnabled = editButtonEnabled;
		removeEnabled = removeButtonEnabled;
		addButton.setEnabled(addButtonEnabled);
		if (fShowEdit) editButton.setEnabled(editButtonEnabled);
		removeButton.setEnabled(removeButtonEnabled);
	}
	
	
	private void createButtons(TeamFormPart teamFormPart, Composite parent, final TableViewer tableViewer)
	{
		final Composite buttonBar = new Composite(parent, SWT.NONE);
		
		final GridData gridData = new GridData(SWT.LEFT, SWT.BEGINNING, false, false);
		buttonBar.setLayoutData(gridData);
		
		GridLayoutFactory.fillDefaults().numColumns(fBelowTable ? 3 : 1).equalWidth(true).margins(0, 5).applyTo(buttonBar);
		if (!fBelowTable) ((GridLayout)buttonBar.getLayout()).marginTop = tableViewer.getTable().getHeaderHeight();
		
		final FormToolkit toolkit = teamFormPart.getSite().getToolkit();
		
		//create buttons
		addButton = createButton(buttonBar, Messages.ButtonAndContextMenu_ADD,  toolkit, true);
		
		addButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				addEditRemoveUi.addEvent();
			}
		});
		
		if (fShowEdit) {
			editButton = createButton(buttonBar, Messages.ButtonAndContextMenu_EDIT, toolkit, false);
			editButton.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e)
				{
					addEditRemoveUi.editEvent();
				}
			});
		}
		
		removeButton = createButton(buttonBar, Messages.ButtonAndContextMenu_REMOVE, toolkit, false);
		removeButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				addEditRemoveUi.removeEvent();
			}
		});
		
		tableViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) 
			{
				final IStructuredSelection selection = (IStructuredSelection) tableViewer.getSelection();
				removeButton.setEnabled(!selection.isEmpty() && removeEnabled);
				if (fShowEdit) editButton.setEnabled(selection.size() == 1);
			}
		});
		
		if (fShowEdit) {
			tableViewer.addDoubleClickListener(new IDoubleClickListener() {
				public void doubleClick(DoubleClickEvent event) 
				{
					final IStructuredSelection selection = (IStructuredSelection) tableViewer.getSelection();
					
					if (selection.size() == 1) 
					{
						addEditRemoveUi.editEvent();
					}
				}
			});
		}
	}
	
	
	private void createContextMenu(final TableViewer tableViewer)
	{
		final MenuManager contextMenuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		contextMenuMgr.setRemoveAllWhenShown(true);
		contextMenuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(final IMenuManager manager) {
				IStructuredSelection selection = (IStructuredSelection) tableViewer.getSelection();
				
				final Action add = new Action(Messages.ButtonAndContextMenu_ADD){
					public void run() {
						addEditRemoveUi.addEvent();
					}
				};
				add.setEnabled(addEnabled);
				contextMenuMgr.add(add);
				
				if (fShowEdit && selection.size() == 1)
				{
					final Action edit = new Action(Messages.ButtonAndContextMenu_EDIT) {
						public void run() {
							addEditRemoveUi.editEvent();
						}
					};
					edit.setEnabled(editEnabled);
					contextMenuMgr.add(edit);
				}
				
				if (!selection.isEmpty())
				{
					final Action remove = new Action(Messages.ButtonAndContextMenu_REMOVE) {
						public void run() {
							addEditRemoveUi.removeEvent();
						}
					};

					remove.setEnabled(removeEnabled);
					contextMenuMgr.add(remove);
				}				
			}
		});
		
		tableViewer.getTable().setMenu(contextMenuMgr.createContextMenu(tableViewer.getControl()));
	}	
	
	private Button createButton(Composite buttonBar, String Label, FormToolkit toolkit, boolean enabled)
	{
		final Button button = toolkit.createButton(buttonBar, Label, SWT.FLAT);
		final GridData buttonGridData = new GridData(SWT.FILL, SWT.CENTER, false, false);
		buttonGridData.minimumWidth = 50;
		
		button.setLayoutData(buttonGridData);
		button.setEnabled(enabled);
		
		return button;
	}
}
