/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.workitem.ide.ui.internal.aspecteditor;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.resource.LocalResourceManager;
import org.eclipse.jface.resource.ResourceManager;

import com.ibm.team.foundation.common.util.IMemento;
import com.ibm.team.process.client.workingcopies.IProcessContainerWorkingCopy;
import com.ibm.team.process.common.ModelElement;
import com.ibm.team.process.common.ModelElements;
import com.ibm.team.process.ide.ui.AbstractConfigurationDataAspectEditor;
import com.ibm.team.process.ide.ui.IProcessAspectEditorSite;
import com.ibm.team.process.ide.ui.ProcessAspect;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.AbstractWorkItemAspectEditor.IAspectMessageProvider;

/**
 * Base class for WI aspect editors that want to push back to the process XML
 * but don't show a combo at the top of the editor. This shares much with {@link AbstractWorkItemAspectEditor}
 * with the exception of the combo. That class should probably be made to extend this to reduce code duplication.
 * <p>
 * This class mainly helps with managing an array of modeled elements that are later serialized to XML.
 */
public abstract class BasicWorkItemAspectEditor
	extends AbstractConfigurationDataAspectEditor
	implements IDirtyStateTracker {
	private List<ModeledElement> fElements;
	private final String fConfigurationDataId;
	private String fHelpContextId= null;

	private ResourceManager fResourceManager= new LocalResourceManager(JFaceResources.getResources());

	public BasicWorkItemAspectEditor(String dataId) {
		fConfigurationDataId= dataId;
	}
	
	public IProcessContainerWorkingCopy getProcessContainerWorkingCopy() {
		return getAspect().getProcessContainerWorkingCopy();
	}

	protected void setHelpContextId(String helpContextId) {
		fHelpContextId= helpContextId;
	}

	@Override
	public void init(IProcessAspectEditorSite site, ProcessAspect inputAspect) {
		super.init(site, inputAspect);
		init();
	}
	
	private void init() {
		fElements= new ArrayList<ModeledElement>(readElements(getAspect().getConfigurationElement()));
	}
	
	protected abstract IAspectMessageProvider getMessageProvider();

	/**
	 * The configuration element may be <code>null</code> (nothing configured yet).
	 */
	protected abstract List<? extends ModeledElement> readElements(ModelElement configurationElement);

	protected void addElement(ModeledElement el) {
		fElements.add(el);
		setDirty();
	}
	
	@Override
	public final boolean saveState(IMemento memento) {
		memento.putString(ModelElements.CONFIGURATION_DATA_ID_ATTRIBUTE, fConfigurationDataId);
		String namespace = getAspect().getSchemaNamespaceURI();
		if (namespace != null) {
			memento.putString("xmlns", namespace); //$NON-NLS-1$
		}
		commit(fElements, memento);
		return true;  // TODO: should probably return false to delete the configuration element if fElements is empty
	}
	
	/**
	 * @param elements The complete list of elements
	 * @param memento The memento to commit the state into
	 */
	protected abstract void commit(List<? extends ModeledElement> elements, IMemento memento);

	/**
	 * Should only be used read-only.
	 */
	protected List<? extends ModeledElement> getAllElements() {
		return fElements;
	}
	
	@Override
	public final void dispose() {
		doDispose();
		if (fResourceManager != null) {
			fResourceManager.dispose();
			fResourceManager= null;
		}
	}

	protected ResourceManager getResourceManager() {
		return fResourceManager;
	}
	
	/**
	 * @return Whether the implementing aspect editor can validate model elements. Defaults to <code>false</code>
	 */
	protected boolean canValidate() {
		return false;
	}

	/**
	 * @param newElement may be null if no elements exist at all
	 */
	protected abstract void inputChanged(ModeledElement newElement);

	protected String getValidateLinkLabel() {
		return ""; //$NON-NLS-1$
	}

	protected void validate(ModeledElement element) {
		
	}

	@Override
	public boolean needsApplyAndRevertButtons() {
		return false;
	}
	
	protected abstract void doDispose();

}
