/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import com.ibm.team.workitem.ide.ui.internal.aspecteditor.enumeration.EnumerationPart;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.quickinformation.QuickinformationAspectPart;

/**
 * TableViewerReorderingSupport is a support class that implements all the dirty details of
 * creating and managing the drag, drop, keyboard and mouse listeners that allow a TableViewer
 * to have drag and drop support for re ordering its elements. This class also enables key board
 * short cuts Ctrl+Alt+Up and Ctrl+Alt+Down to move selected items in the table up or down
 * using the keyboard for accessibility reasons. This class must be passed a TableViewer that
 * has its own backing data structure that implements the {@link IReorderableList} interface.
 * The TableViewer is updated with all the listeners and the backing data structure is modified
 * to reorder the list.
 * 
 * @author jsteele
 */
public class TableViewerReorderingSupport {
	
	final private static int OPERATIONS = DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_DEFAULT;
	final private static String DELIMITER = ","; //$NON-NLS-1$
	
	private final TableViewer tableViewer;
	private final IDirtyStateTracker dirtyTracker;
	private IReorderableList reorderableList;
	
	
	/**
	 * Construct a new TableInternalDragAndDropSorting to allow sorting of items
	 * within a TableViewer either by using drag and drop or keyboard shortcuts.
	 * 
	 * @param moveableEntries
	 *            A class that contains a specific type of entry that can be
	 *            selected and sorted. For example the {@link EnumerationPart}
	 *            or {@link QuickinformationAspectPart} classes.
	 * @param tableViewer
	 *            A normal TableViewer class to add the drag, drop, mouse and
	 *            keyboard listeners too.
	 */
	public TableViewerReorderingSupport(IDirtyStateTracker dirtyTracker, TableViewer tableViewer) {
		this.dirtyTracker = dirtyTracker;
		this.tableViewer = tableViewer;
	}
	
	
	/**
	 * Actually create and initialize all the listeners for the table (drag,
	 * drop, mouse and keyboard). Without calling this method no support for any
	 * of these operations will be enabled.
	 */
	public void initializeDragAndDropSupport() {
		final Transfer[] transfers = new Transfer[] { TextTransfer.getInstance() };
		
		addDragSupport(transfers);
		addDropSupport(transfers);
		addMouseListenerSupport();
		addKeyListenerSupport();
	}
	
	
	/**
	 * Provides this class with a class that implements the IReorderableList
	 * interface. The IReorderableList must be the class that contains the
	 * backing data for the provided TableViewer. Without calling this method no
	 * reordering will take place.
	 * 
	 * @param reorderableList
	 *            A class that implements the IReorderableList list interface.
	 */
	public void setReorderableList(IReorderableList reorderableList) {
		this.reorderableList = reorderableList;
	}
	
	
	private void addDragSupport(Transfer[] transfers) {
		tableViewer.addDragSupport(OPERATIONS, transfers, new DragSourceAdapter() {
			public void dragStart(DragSourceEvent event) {
				final List<Object> moveElements = getSelectedEntries();
				
				if (moveElements != null && !moveElements.isEmpty()) {
					event.doit = true;
				}
				else {
					event.doit = false;
				}
			}
			
			
			public void dragFinished(DragSourceEvent event) {
				// Nothing to do on drag finished.
			}
			
			
			public void dragSetData(DragSourceEvent event) {
				if (reorderableList == null) {
					event.doit = false;
					event.data = ""; //$NON-NLS-1$
					return;
				}
				
				final List<Object> moveObjects = getSelectedEntries();
				final StringBuilder commaDelimitedIdList = new StringBuilder();
				
				
				// Build up a comma delimited list of IDs for the entries to be moved.
				// All the entries in the IReorderable list must have an ID for this to work.
				for (Object object : moveObjects) {
					if (commaDelimitedIdList.length() > 0) {
						commaDelimitedIdList.append(DELIMITER);
					}
					
					commaDelimitedIdList.append(reorderableList.getObjectsId(object));
				}
				
				event.data = commaDelimitedIdList.toString();
			}
		});
	}
	
	
	private void addDropSupport(Transfer[] transfers) {
		tableViewer.addDropSupport(OPERATIONS, transfers, new DropTargetAdapter() {
			public void drop(DropTargetEvent event) {
				if (TextTransfer.getInstance().isSupportedType(event.currentDataType)) {
					final String commaDelimitedIdList = (String) event.data;
					final List<String> ids = Arrays.asList(commaDelimitedIdList.split(DELIMITER));
					
					TableItem item = null;
					
					// Get the item the user performed the drop on in the table.
					if (event.item instanceof TableItem) {
						item = (TableItem) event.item;
					}
					
					moveIdsAfterItem(ids, item);
				}
			}
			
			public void dragOver(DropTargetEvent event) {
					event.feedback = DND.FEEDBACK_SELECT | DND.FEEDBACK_SCROLL | DND.FEEDBACK_INSERT_AFTER;
			}
				
			public void dragEnter(DropTargetEvent event) {
				dragOver(event);
				
				// Allow dropping text only
				for (TransferData transferData : event.dataTypes) {
					if (TextTransfer.getInstance().isSupportedType(transferData)) {
						event.currentDataType = transferData;
					}
				}
			}
		});
	}
	
	
	private void addMouseListenerSupport() {
		tableViewer.getTable().addListener(SWT.MouseMove, new Listener() {
			public void handleEvent(Event event) {
				updateMouseCursor(event);
			}
		});
		
		tableViewer.getTable().addListener(SWT.MouseUp, new Listener() {
			public void handleEvent(Event event) {
				updateMouseCursor(event);
			}
		});
	}
	
	
	private void updateMouseCursor(Event event) {
		if (reorderableList == null) {
			return;
		}
		
		final Point point = new Point(event.x, event.y);
		final Table table = (Table) event.widget;
		final TableItem item = table.getItem(point);
		
		if (item == null)
			return;
		
		final TableItem[] selectedItems = table.getSelection();
		boolean itemIsSelected = false;
		
		// Determine if the item the user is hovering over or clicked down on
		// is one of the selected items.
		for (TableItem selectedItem : selectedItems) {
			if (selectedItem.equals(item)) {
				itemIsSelected = true;
				break;
			}
		}
		
		// If a selected item and the item is in the backing list then
		// set the hand cursor otherwise set the arrow cursor.
		if (itemIsSelected && reorderableList.indexOf(item.getData()) != -1) {
			table.setCursor(table.getDisplay().getSystemCursor(SWT.CURSOR_HAND));
		}
		else {
			table.setCursor(table.getDisplay().getSystemCursor(SWT.CURSOR_ARROW));
		}
	}
	
	
	private void addKeyListenerSupport() {
		tableViewer.getTable().addListener(SWT.KeyDown, new Listener() {
			public void handleEvent(Event event) {
				// Check that CTRl and ALT are both pressed, then check if the Up or Down key was pressed.
				if (((event.stateMask & SWT.CTRL) != 0) && ((event.stateMask & SWT.ALT) != 0)) {
					if (event.keyCode == SWT.ARROW_UP) {
						moveSelectedItemsUp();
					}
					else if (event.keyCode == SWT.ARROW_DOWN) {
						moveSelectedItemsDown();
					}
				}
			}
		});
	}
	
	
	private void updateTableSelection(final List<?> selectedEntries) {
		Display.getDefault().asyncExec(new Runnable() {
			public void run() {
				tableViewer.refresh();
				// Make sure that the currently selected items stay selected after the move.
				tableViewer.setSelection(new StructuredSelection(selectedEntries), true);
			}
		});
	}
	
	
	private List<Object> getSelectedEntries() {
		if (reorderableList == null) {
			return Collections.emptyList();
		}
		
		final List<Object> selectedObjects = new ArrayList<Object>();
		
		if (tableViewer.getSelection() instanceof IStructuredSelection) {
			final IStructuredSelection selection = (IStructuredSelection) tableViewer.getSelection();
			
			for (Object object : selection.toList()) {
				if (reorderableList.indexOf(object) != -1) {
					selectedObjects.add(object);
				}
			}
		}
		
		return selectedObjects;
	}
	
	
	private void moveIdsAfterItem(List<String> ids, TableItem item) {
		if (reorderableList == null) {
			return;
		}
		
		final List<Object> currentObjects = reorderableList.getCurrentObjects();
		Object dropAfterObject = item != null ? item.getData() : null;
		
		// If the item the user was hovering over in the table is
		// null then move the selected items to the very end.
		if (dropAfterObject == null) {
			if (currentObjects.isEmpty()) {
				return;
			}
			
			dropAfterObject = currentObjects.get(currentObjects.size() - 1);
		}
		
		final List<Object> moveObjects = new ArrayList<Object>();
		
		// Loop over all the current objects to find where the IDs match
		// the items that are supposed to move.
		for (Object object : currentObjects) {
			for (String id : ids) {
				if (id.equals(reorderableList.getObjectsId(object))) {
					moveObjects.add(object);
					break;
				}
			}
		}
		
		// If they are trying to move everything or nothing then do nothing.
		if (moveObjects.size() == currentObjects.size() || moveObjects.size() == 0) {
			return;
		}
		
		int moveAfterPosition = currentObjects.indexOf(dropAfterObject);
		boolean movingUp;
		
		if (moveAfterPosition == -1) {
			moveAfterPosition = currentObjects.size() - 1;
			movingUp = false;
		}
		else {
			int lastMoveIndex = reorderableList.indexOf(moveObjects.get(moveObjects.size() - 1));
			
			if (moveAfterPosition > lastMoveIndex) {
				movingUp = false;
			}
			else {
				movingUp = true;
			}
		}
		
		if (movingUp) {
			for (int i = moveObjects.size() - 1; i >= 0; i--) {
				Object object = moveObjects.get(i);
				reorderableList.moveTo(moveAfterPosition, object);
			}
		}
		else {
			for (Object object : moveObjects) {
				reorderableList.moveTo(moveAfterPosition, object);
			}
		}
		
		updateTableSelection(moveObjects);
		dirtyTracker.setDirty();
	}
	
	
	/**
	 * Moves the items that are currently selected in the TableViewer up in the
	 * list if it is possible.
	 */
	public void moveSelectedItemsUp() {
		if (reorderableList == null) {
			return;
		}
		
		final List<Object> moveObjects = getSelectedEntries();
		
		if (moveObjects.isEmpty())
			return;
		
		// Get the index of the first object
		int current = reorderableList.indexOf(moveObjects.get(0));
		
		if (current == 0)
			return;
		
		// Move the selected objects to the current position -1 and add one
		// each iteration.
		for (Object object : moveObjects) {
			reorderableList.moveTo(current - 1, object);
			current++;
		}
		
		updateTableSelection(moveObjects);
		dirtyTracker.setDirty();
	}
	
	/**
	 * Moves the items that are currently selected in the TableViewer down in
	 * the list if it is possible.
	 */
	public void moveSelectedItemsDown() {
		if (reorderableList == null) {
			return;
		}
		
		final List<Object> moveObjects = getSelectedEntries();
		
		if (moveObjects.isEmpty())
			return;
		
		final List<Object> currentObjects = reorderableList.getCurrentObjects();
		
		// Get the index of the last selected object
		int current = currentObjects.indexOf(moveObjects.get(moveObjects.size() - 1));
		
		if (current == currentObjects.size() - 1) {
			return;
		}
		
		// We want to move the items to the position of the last element + 1.
		int moveToLocation = current + 1;
		
		// Walk over the list backwards so we can keep inserting items into the same spot
		// pushing everything else down.
		for (int i = moveObjects.size() - 1; i >=0; i--) {
			Object object = moveObjects.get(i);
			reorderableList.moveTo(moveToLocation, object);
			moveToLocation--;
		}
		
		updateTableSelection(moveObjects);
		dirtyTracker.setDirty();
	}
}
