/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.resource.ResourceManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import com.ibm.team.jface.internal.ImagePool;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.ProcessAttachmentIconSelectionDialog.IDialogValidator;

/**
 * Base class for dialog that want to show a status (i.e., an icon and message).
 * The status is hidden as long a the dialog validates as <code>Status.OK_STATUS</code>
 * though space is always reserved for it.
 * <p>
 * Sub-class must set a {@link #setValidator(IDialogValidator) validator} to and manually
 * call {@link ValidationDialog#validate()} method at the appropriate time. Additionally,
 * when creating the content of the dialog, a call must be made to {@link ValidationDialog#createErrorStatusDialogPart(Composite)}--
 * this is not called for you (though probably should be...).
 */
public abstract class ValidationDialog extends Dialog {

	private Composite fErrorPart;
	private Label fErrorText;
	private Label fErrorIcon;
	private IDialogValidator fValidator;
	private final ResourceManager fResourceManager;

	/**
	 * @param parentShell
	 */
	public ValidationDialog(Shell parentShell, ResourceManager resourceManager) {
		super(parentShell);
		fResourceManager = resourceManager;
	}
	
	/**
	 * Checks if the dialog is valid, but doesn't show or hide the
	 * status message.
	 * @return
	 * 	current status. OK is there are no validation problems.coC
	 */
	public IStatus isValid() {
		return fValidator == null
			? Status.OK_STATUS
			: fValidator.isValid();
	}
	
	/**
	 * Uses the current validator to validate the dialog.
	 * If the status returned is no "OK", then the error icon
	 * and message are shown for the status.
	 */
	public void validate() {
		if (fErrorText != null && !fErrorText.isDisposed()) {
			IStatus status= Status.OK_STATUS;
			if (fValidator != null) {
				status = fValidator.isValid();
			}
			if (status.getSeverity() == IStatus.ERROR) {
				fErrorText.setText(status.getMessage());
				fErrorText.getParent().getParent().layout();
				fErrorIcon.setImage(fResourceManager.createImage(ImagePool.ERROR_ICON));
				Button button = getButton(IDialogConstants.OK_ID);
				if (button != null) {
					button.setEnabled(false);
				}
			} else if (status.getSeverity() == IStatus.WARNING) {
				fErrorText.setText(status.getMessage());
				fErrorText.getParent().getParent().layout();
				fErrorIcon.setImage(fResourceManager.createImage(ImagePool.WARN_ICON));
				Button button = getButton(IDialogConstants.OK_ID);
				if (button != null) {
					button.setEnabled(true);
				}
			} else if (status.getSeverity() == IStatus.INFO) {
				fErrorText.setText(status.getMessage());
				fErrorText.getParent().getParent().layout();
				fErrorIcon.setImage(fResourceManager.createImage(ImagePool.INFO_ICON));
				Button button = getButton(IDialogConstants.OK_ID);
				if (button != null) {
					button.setEnabled(true);
				}
			} else {
				fErrorText.setText(""); //$NON-NLS-1$
				fErrorText.getParent().getParent().layout();
				fErrorIcon.setImage(null);
				Button button = getButton(IDialogConstants.OK_ID);
				if (button != null) {
					button.setEnabled(true);
				}
			}
		}
	}

	/**
	 * Creates the area for a status icon an message. Normally called
	 * with the content composite for the main area of the dialog after the other controls
	 * have been created to ensure the error message is at the bottom.
	 * @param parent
	 * 	control to add the error icon and message to.
	 * @return
	 * 	the created error part. Can be used to change to layout data
	 * 	if necessary, but normally ignored.
	 */
	protected Composite createErrorStatusDialogPart(Composite parent) {
		GridLayout layout;
		fErrorPart = new Composite(parent, SWT.NONE);
		layout= new GridLayout(2, false);
		layout.marginWidth= 0;
		layout.marginHeight= 0;
		layout.horizontalSpacing= 2;
		fErrorPart.setLayout(layout);
		fErrorPart.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
		fErrorIcon = new Label(fErrorPart, SWT.NONE);
		GridData data= new GridData(SWT.FILL, SWT.TOP, false, false);
		data.widthHint= 18;
		fErrorIcon.setLayoutData(data);
		
		fErrorText= new Label(fErrorPart, SWT.WRAP);
		fErrorText.setText(""); //$NON-NLS-1$
		fErrorText.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		
		return fErrorPart;
	}

	
	/**
	 * Set the validator to use.
	 * Setters have to ensure themselves, that <code>validate</code>
	 * is called in the appropriate places. Whenever the icon changes,
	 * <code>iconChanged</code> is called.
	 */
	public void setValidator(IDialogValidator validator) {
		IDialogValidator old= fValidator;
		fValidator= validator;
		if (old != null) {
			validate();
		}
	}
	
	/**
	 * Hides the validation error area if not needed.
	 * This was added to make use of the ProcessAttachmentIconSelectionDialog (which normally
	 * validates) to pick an icon for a work item type when no validation was needed. In the future
	 * it would probably be cleaner to just break out the icon pieces from that dialog to 
	 * reused them more easily without the validation parts, but this works fine for now. 
	 */
	protected void hideErrorPart() {
		fErrorPart.setVisible(false);
		GridDataFactory.fillDefaults().hint(0, 0).applyTo(fErrorPart);
	}
}