/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.resource.LocalResourceManager;
import org.eclipse.jface.resource.ResourceManager;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.services.IDisposable;

import com.ibm.team.jface.JazzResources;
import com.ibm.team.process.ide.ui.IProcessConfigurationPage;
import com.ibm.team.process.internal.ide.ui.editors.form.DataAspect;
import com.ibm.team.process.internal.ide.ui.editors.form.ProjectConfigurationAspect;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.workitem.common.internal.BindingTarget;
import com.ibm.team.workitem.common.internal.presentations.AbstractPresentationDescriptor;
import com.ibm.team.workitem.common.internal.presentations.EditorPresentationManager;
import com.ibm.team.workitem.common.internal.presentations.SectionDescriptor;
import com.ibm.team.workitem.common.internal.presentations.TabDescriptor;
import com.ibm.team.workitem.ide.ui.internal.ImagePool;
import com.ibm.team.workitem.ide.ui.internal.WorkItemIDEUIPlugin;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.AspectEditorUtil;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.IDirtyStateTracker;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.EditorIdBinding.ReadOnlyType;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.type.TypeAspectEditor;
import com.ibm.team.workitem.ide.ui.internal.editor.DecoratedCombo;
import com.ibm.team.workitem.ide.ui.internal.editor.DecoratedControl;

/**
 * Manages the control for modifying editor ID bindings of work item types.
 * Not managed; must be explicitly created and disposed.
 */
public class EditorIdBindingBlock implements IDisposable {
	
	private ResourceManager fResourceManager = new LocalResourceManager(JFaceResources.getResources());
	
	private final FormToolkit fToolkit;
	private final IDirtyStateTracker fTracker;
	private final TypeAspectEditor fEditor;
	
	private static String NO_ELEMENTS = Messages.EditorIdBindingAspectEditor_NO_ELEMENTS;
	private static final String NONE= NLS.bind(Messages.DEFAULT, EditorPresentationManager.DEFAULT_EDITOR);
	private Map<BindingTarget, DecoratedCombo> fCombos = new HashMap<BindingTarget, DecoratedCombo>();

	private EditorIdBinding.ReadOnlyType fType;
	
	public EditorIdBindingBlock(
			FormToolkit toolkit,
			IDirtyStateTracker tracker,
			TypeAspectEditor editor) {
		fToolkit = toolkit;
		fTracker = tracker;
		fEditor = editor;
	}
	
	public void createContent(Composite parent) {
		final Composite clientArea = fToolkit.createComposite(parent);
		GridDataFactory.fillDefaults().span(2, 1).grab(false, true).applyTo(clientArea);
		final FormLayout formLayout = new FormLayout();
		clientArea.setLayout(formLayout);
		
		Control previous = null;
		
		for (final BindingTarget t : fEditor.getTargets()) {
			
			Composite comp = fToolkit.createComposite(clientArea);
			comp.setLayoutData(new GridData(SWT.BEGINNING, SWT.BEGINNING,true,false));
			GridLayout gLayout = new GridLayout();
			gLayout.verticalSpacing = 0;
			gLayout.marginWidth = 0;
			comp.setLayout(gLayout);
			
			Label l = fToolkit.createLabel(comp, t.getDisplayName(), SWT.WRAP);
			if (null != t.getToolTipText()) {
				l.setToolTipText(t.getToolTipText());
			}

			Composite comboComp = fToolkit.createComposite(comp, SWT.NONE);
			GridDataFactory.fillDefaults().grab(true, false).applyTo(comboComp);
			GridLayout cmbLayout = new GridLayout();
			cmbLayout.numColumns = 2;
			cmbLayout.marginWidth = 0;
			comboComp.setLayout(cmbLayout);
			final DecoratedCombo decoratedCombo = new DecoratedCombo(comboComp, SWT.READ_ONLY, DecoratedControl.IMAGE_OPTION);
			fToolkit.adapt(decoratedCombo.getCombo(), true, false);
			GridDataFactory.fillDefaults().grab(true, false).applyTo(decoratedCombo.getLayoutControl());
			GridDataFactory.fillDefaults().grab(true, false).applyTo(decoratedCombo.getCombo());

			decoratedCombo.setLabelProvider(new LabelProvider() {
				public String getText(Object element) {
					return (String) element;
				}

				public Image getImage(Object element) {
					if (EditorPresentationManager.TAB.equals(t.getElementType())) {
						return JazzResources.getImage(fResourceManager, ImagePool.TAB, null);
					} else if (EditorPresentationManager.SECTION.equals(t.getElementType())) {
						return JazzResources.getImage(fResourceManager, ImagePool.SECTION, null);
					} else if (EditorPresentationManager.EDITOR.equals(t.getElementType())) {
						return JazzResources.getImage(fResourceManager, ImagePool.EDITOR, null);
					}
					return null;
				}
			});

			final ToolBar toolBar = new ToolBar(comboComp, SWT.FLAT);
			fToolkit.adapt(toolBar);
			final ToolItem editItem = new ToolItem(toolBar, SWT.PUSH);
			editItem.setImage(JazzResources.getImageWithDefault(fResourceManager, com.ibm.team.workitem.rcp.ui.internal.ImagePool.EDIT_QUERY_ACTION_ICON));
			editItem.setToolTipText(com.ibm.team.workitem.ide.ui.internal.aspecteditor.Messages.ButtonAndContextMenu_EDIT);
			 editItem.addSelectionListener(new SelectionListener() {
							
				 final DecoratedCombo fCombo = decoratedCombo;
								
				 public void widgetSelected(SelectionEvent e) {

					 IEditorPart part =PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
					 if(part instanceof FormEditor){
						 FormEditor pEditor = (FormEditor)part;
						 IProcessConfigurationPage pPage = (IProcessConfigurationPage)pEditor.getActivePageInstance();
						 pPage.selectAndRevealAspect(constructEditorPresentationAspectPath());
						 EditorPresentationAspectEditor presEditor = (EditorPresentationAspectEditor)pPage.getCurrentAspectEditor();
						 
						 String selection = (String)((StructuredSelection)fCombo.getSelection()).getFirstElement();
						 EditorPresentation presentation = null;
						 List<Object> treePath = new ArrayList<Object>();
						 /**
						  * Link is an Editor, set presentation selection to the Editor
						  */
						 if(EditorPresentationManager.EDITOR.equals(t.getElementType())){
							 if(selection.equals(NONE)){
								 selection = EditorPresentationManager.DEFAULT_EDITOR;
							 }
							 presentation = new EditorPresentation(selection); 
							 /**
							  * Link is a tab, find a presentation editor that contains the tab and select the tab in the tree.
							  */
						 } else if(EditorPresentationManager.TAB.equals(t.getElementType())){
							 if(!selection.equals(NONE)){
								Set<String/*editor id*/> editors =  presEditor.getEditorDependencyCache().get(selection);
								if(editors.size() > 0 ){
									presentation =  new EditorPresentation(editors.iterator().next());
									List<AbstractPresentationDescriptor> descs = presEditor.getPresentationsMap().get(presentation.getIdentifier());
									for (AbstractPresentationDescriptor desc : descs) {
										if (desc instanceof TabDescriptor) {
											if(selection.equals(desc.getElementId())){
												treePath.add(desc);
												break;
											}
										}
									}
								} else {
									String msg = NLS.bind(Messages.EditorIdBindingBlock_NO_EDITOR_FOR_TAB,selection);
									ErrorDialog.openError(null, Messages.EditorIdBindingBlock_NO_EDITOR_DIALOG_TITLE, msg, new Status(Status.WARNING, WorkItemIDEUIPlugin.PLUGIN_ID, msg));
								}
								
							 }
							 /**
							  * Link is a section, find a presentation editor that contains the section and select it in the tree.
							  */
						 } else if(EditorPresentationManager.SECTION.equals(t.getElementType())){
							 if(!selection.equals(NONE)){
								Set<TabDescriptor> tabs = presEditor.getSectionDependencyCache().get(selection);
//								List<String> editors = new ArrayList<String>();
								TabDescriptor tabDescriptor = tabs.iterator().next();
//								for (TabDescriptor tabDescriptor : tabs) {
								Set<String/*editor id*/> tabEditors = presEditor.getEditorDependencyCache().get(tabDescriptor.getElementId());
//								editors.addAll(tabEditors);
//								}
								if(tabEditors.size() > 0){
									presentation =  new EditorPresentation(tabEditors.iterator().next());
									TabDescriptor tab = tabs.iterator().next();
									treePath.add(tab);
									List<AbstractPresentationDescriptor> descs = presEditor.getPresentationsMap().get(tab.getElementId());
									for (AbstractPresentationDescriptor desc : descs) {
										if (desc instanceof SectionDescriptor) {
											if(selection.equals(desc.getElementId())){
												treePath.add(desc);
												break;
											}
										}
									}
								} else {
									String msg = NLS.bind(Messages.EditorIdBindingBlock_NO_EDITOR_FOR_SECTION,selection);
									ErrorDialog.openError(null, Messages.EditorIdBindingBlock_NO_EDITOR_DIALOG_TITLE, msg, new Status(Status.WARNING, WorkItemIDEUIPlugin.PLUGIN_ID, msg));
								}
								
							 }						 
						 }
						 /**
						  * Set selection in combo and tree in presentation editor.
						  */
						 if(presentation != null){
							presEditor.setComboInput(presentation);
							if(treePath.size() > 0){
								TreeSelection tselection = new TreeSelection(new TreePath(treePath.toArray()));
								presEditor.selectInTree(tselection);
							}
						 }

					 }
									
				 }
								
				 public void widgetDefaultSelected(SelectionEvent e) {
					 widgetSelected(e);
				 }
							
			 });
			 
			 decoratedCombo.addSelectionChangedListener(new ISelectionChangedListener() {
					public void selectionChanged(SelectionChangedEvent event) {
						String value = (String) decoratedCombo.getValue();
						if (Messages.EditorIdBindingAspectEditor_NONE_ELEMENT.equals(value) || NONE.equals(value)) {
							editItem.setEnabled(NONE.equals(value) && fType != null);
							value = null;
						} else {
							editItem.setEnabled(fType != null);
						}
						EditorIdBinding binding = findBinding(t, fType);
						
						if (binding != null) {
							if (!AspectEditorUtil.equals(value, binding.getEditorId())) {
								binding.setEditorId(value);
								setDirty();
							}
						}
					}
			 });
			
			FormData frmData = new FormData();

			if (previous == null) {
				frmData.top = new FormAttachment(0, 7);
			} else {
				frmData.top = new FormAttachment(previous);
			}
			previous = comp;

			frmData.left = new FormAttachment(0);
			frmData.right = new FormAttachment(100);

			comp.setLayoutData(frmData);
			
			fCombos.put(t, decoratedCombo);

		}
	}

	public void setInput(Object input) {
		if (input instanceof ReadOnlyType) {
			fType = (ReadOnlyType) input;
		} else {
			fType = null;
		}
		
		updateComboValues();
	}

	private List<String> findValues(BindingTarget target) {
		if (EditorPresentationManager.TAB.equals(target.getElementType())) {
			return fEditor.getTabIds();
		} else if (EditorPresentationManager.EDITOR.equals(target.getElementType())) {
			return fEditor.getEditorIds();
		} else if (EditorPresentationManager.SECTION.equals(target.getElementType())) {
			return fEditor.getSectionIds();
		}
		return Collections.emptyList();
	}

	private void updateComboValues() {

		for (BindingTarget t : fEditor.getTargets()) {
			DecoratedCombo combo = fCombos.get(t);
			if (combo == null) {
				continue;
			}
			List<String> values = new ArrayList<String>(findValues(t));
			if (values.isEmpty()) {
				values.add(NO_ELEMENTS);
				combo.getCombo().setEnabled(false);
			} else {
				values.add(0, EditorPresentationManager.EDITOR.equals(t.getElementType())?NONE:Messages.EditorIdBindingAspectEditor_NONE_ELEMENT);
				combo.getCombo().setEnabled(true);
			}
			EditorIdBinding current = findBinding(t, fType);
			String value= EditorPresentationManager.EDITOR.equals(t.getElementType())?NONE:Messages.EditorIdBindingAspectEditor_NONE_ELEMENT;
			if (current != null) {
				for (String v : values) {
					if (current.getEditorId() != null && current.getEditorId().equals(v)) {
						value = v;
					}
				}
			}
			combo.getCombo().setEnabled(fType != null);
			combo.getImageControl().setEnabled(fType != null);

			combo.setValueSet(values.toArray());
			combo.setValue(value);
		}
	}
	
	private void setDirty() {
		fTracker.setDirty();
	}

	private EditorIdBinding findBinding(BindingTarget target, ReadOnlyType type) {
		List<EditorIdBinding> bindings = findBindings(target);
		for (EditorIdBinding binding : bindings) {
			if (binding.getType().equals(type)) {
				return binding;
			}
		}
		return null;
	}

	private List<EditorIdBinding> findBindings(BindingTarget target) {
		List<EditorIdBinding> values = new ArrayList<EditorIdBinding>();
		for (EditorIdBinding binding : fEditor.getAllEditorIdBindings()) {
			if (AspectEditorUtil.equals(binding.getTarget(), target.getTargetId())) {
				values.add(binding);
			}
		}
		return values;
	}
	
	public void dispose() {
		fResourceManager.dispose();
	}
	
	/**
	 * Build aspect path to the Editor Presentation aspect
	 * 
	 * @return
	 */

	 private List<Object> constructEditorPresentationAspectPath(){
	 ArrayList<Object> path = new ArrayList<Object>();
	 path.add(ProjectConfigurationAspect.ASPECT_ID);
	 path.add(DataAspect.ASPECT_ID);
			path.add("com.ibm.team.workitem.category"); //$NON-NLS-1$
			path.add("com.ibm.team.workitem.editor.configuration.presentations"); //$NON-NLS-1$
	 return path;
	 }
}
