/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.workitem.java.ide.ui.internal.editor.comments;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.IImportDeclaration;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.Signature;
import org.eclipse.jdt.ui.JavaElementLabels;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ISetSelectionTarget;

import com.ibm.team.foundation.common.URIReference;
import com.ibm.team.foundation.rcp.core.hyperlinks.ContextProvider;
import com.ibm.team.foundation.rcp.core.hyperlinks.HyperlinkHandler;
import com.ibm.team.workitem.java.ide.ui.internal.WorkItemJavaIDEUIPlugin;

import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 * Hyperlink handler for Java elements.
 */
public final class JavaElementHyperlinkHandler extends HyperlinkHandler {

	/** The java handle scheme (value: <code>"javahandle"</code>) */
	public static final String JAVA_HANDLE_SCHEME= "javahandle"; //$NON-NLS-1$

	/** The java line scheme (value: <code>"javaline"</code>) */
	public static final String JAVA_LINE_SCHEME= "javaline"; //$NON-NLS-1$

	/** The java qualified scheme (value: <code>"javaqualified"</code>) */
	public static final String JAVA_QUALIFIED_SCHEME= "javaqualified"; //$NON-NLS-1$

	/**
	 * Creates a new hyperlink URI for the given java element.
	 * 
	 * @param element the java element, which must not be <code>null</code>
	 * @return a hyperlink URI, resolvable by this hyperlink handler
	 */
	public static final URI createURI(final IJavaElement element) {
		Assert.isNotNull(element);
		String identifier= element.getHandleIdentifier();
		if (element instanceof IImportDeclaration) {
			final StringBuilder builder= new StringBuilder(identifier);
			final int index= builder.indexOf("#"); //$NON-NLS-1$
			if (index >= 0)
				builder.replace(index, index + 1, "\\\\#"); //$NON-NLS-1$
			identifier= builder.toString();
		}
		try {
			return new URI(JAVA_HANDLE_SCHEME + "://" + URLEncoder.encode(identifier, "UTF-8")); //$NON-NLS-1$ //$NON-NLS-2$
		} catch (UnsupportedEncodingException e) {
			Assert.isTrue(false);
		} catch (URISyntaxException e) {
			Assert.isTrue(false);
		}
		return null;
	}

	/**
	 * Creates a new hyperlink URI for the given simple type and line number.
	 * <p>
	 * The simple type must be the simple name of a top-level Java type. This is
	 * equal to the compilation unit name without the java-like file extension.
	 * </p>
	 * <p>
	 * This method is intended to create hyperlinks for stacktrace elements and
	 * similar references to elements within a compilation unit.
	 * </p>
	 * 
	 * @param simpleType the simple type name, which must not be <code>null</code>
	 * @param line the line number, or <code>-1</code> if unavailable
	 * @return a hyperlink URI, resolvable by this hyperlink handler
	 */
	public static final URI createURI(final String simpleType, final int line) {
		Assert.isNotNull(simpleType);
		try {
			String string= simpleType;
			if (line >= 0) {
				string= string + ":" + line; //$NON-NLS-1$
			}
			string= URLEncoder.encode("(" + string + ")", "UTF-8"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			string= JAVA_LINE_SCHEME + "://" + string; //$NON-NLS-1$
			return new URI(string);
		} catch (URISyntaxException e) {
			Assert.isTrue(false);
		} catch (UnsupportedEncodingException e) {
			// UTF-8 is assumed to be supported
			Assert.isTrue(false);
		}
		return null;
	}

	/**
	 * Creates a new hyperlink URI for the given qualified type name and method
	 * name.
	 * <p>
	 * The qualified type name must be the fully qualified name of a Java type.
	 * </p>
	 * <p>
	 * The argument type names must be the simple names of the method argument
	 * types.
	 * </p>
	 * <p>
	 * This method is intended to create hyperlinks to test methods or similar
	 * references to Java elements.
	 * </p>
	 * 
	 * @param qualifiedType the fully qualified type name, which must not be
	 * <code>null</code>
	 * @param methodName the method name, which must not be <code>null</code>
	 * @param simpleArgumentTypes the simple names of the argument types, or
	 * <code>null</code> if none
	 * @return a hyperlink URI, resolvable by this hyperlink handler
	 */
	public static final URI createURI(final String qualifiedType, final String methodName, final String[] simpleArgumentTypes) {
		Assert.isNotNull(qualifiedType);
		Assert.isNotNull(methodName);
		try {
			final StringBuilder builder= new StringBuilder();
			if (simpleArgumentTypes != null) {
				for (int index= 0; index < simpleArgumentTypes.length; index++) {
					builder.append('.');
					builder.append(simpleArgumentTypes[index]);
				}
			}
			return new URI(JAVA_QUALIFIED_SCHEME + "://" + qualifiedType + "#" + methodName + builder.toString()); //$NON-NLS-1$ //$NON-NLS-2$
		} catch (URISyntaxException e) {
			Assert.isTrue(false);
		}
		return null;
	}

	/**
	 * Opens the specified java element.
	 * 
	 * @param element the java element to open, which must not be
	 * <code>null</code>
	 * @throws PartInitException if an error occurs while opening the java element
	 * @throws JavaModelException if an error occurs while opening the java
	 * element
	 */
	private static void openElement(final IJavaElement element) throws PartInitException, JavaModelException {
		Assert.isNotNull(element);
		final int type= element.getElementType();
		switch (type) {
			case IJavaElement.JAVA_PROJECT:
			case IJavaElement.PACKAGE_FRAGMENT:
			case IJavaElement.PACKAGE_FRAGMENT_ROOT: {
				final IWorkbenchWindow window= PlatformUI.getWorkbench().getActiveWorkbenchWindow();
				if (window != null) {
					final IWorkbenchPage page= window.getActivePage();
					if (page != null) {
						final IViewPart part= page.showView(JavaUI.ID_PACKAGES);
						if (part instanceof ISetSelectionTarget) {
							final ISetSelectionTarget extended= (ISetSelectionTarget) part;
							extended.selectReveal(new StructuredSelection(element));
						}
					}
				}
				break;
			}
			default: {
				JavaUI.openInEditor(element, true, true);
				break;
			}
		}
	}

	/**
	 * Resolves the specified uri.
	 * 
	 * @param uri the uri to resolve, which must not be <code>null</code>
	 * @return the corresponding java element, or <code>null</code> if unable to
	 * resolve the uri
	 */
	private static IJavaElement resolveHandle(final String uri) {
		Assert.isNotNull(uri);
		final int index= uri.indexOf("://"); //$NON-NLS-1$
		if (index >= 0) {
			try {
				final StringBuilder builder= new StringBuilder(URLDecoder.decode(uri.substring(index + 3), "UTF-8")); //$NON-NLS-1$
				final int offset= builder.indexOf("\\\\#"); //$NON-NLS-1$
				if (offset >= 0)
					builder.replace(offset, offset + 5, "#"); //$NON-NLS-1$
				return JavaCore.create(builder.toString());
			} catch (UnsupportedEncodingException exception) {
				Assert.isTrue(false);
			}
		}
		return null;
	}

	/**
	 * Resolves the specified uri.
	 * <p>
	 * Clients may test the uri to resolve by calling {@link #handles(URI)} first.
	 * </p>
	 * 
	 * @param uri the uri to resolve, which must not be <code>null</code>
	 * @return the corresponding java element, or <code>null</code> if unable to
	 * resolve the uri
	 */
	public static IJavaElement resolveJavaElement(final URI uri) {
		Assert.isNotNull(uri);
		final String scheme= uri.getScheme();
		final String string= uri.toString();
		if (JAVA_HANDLE_SCHEME.equals(scheme))
			return resolveHandle(string);
		else if (JAVA_LINE_SCHEME.equals(scheme))
			return resolveLine(string);
		else if (JAVA_QUALIFIED_SCHEME.equals(scheme))
			return resolveQualified(string);
		return null;
	}

	/**
	 * Resolves the specified uri.
	 * 
	 * @param uri the uri to resolve, which must not be <code>null</code>
	 * @return the corresponding java element, or <code>null</code> if unable to
	 * resolve the uri
	 */
	private static IJavaElement resolveLine(final String uri) {
		Assert.isNotNull(uri);
		String string= null;
		try {
			string= URLDecoder.decode(uri.toString(), "UTF-8"); //$NON-NLS-1$
		} catch (UnsupportedEncodingException e1) {
			// UTF-8 is assumed to be supported
			Assert.isTrue(false);
		}
		final int index= string.indexOf("://"); //$NON-NLS-1$
		if (index >= 0) {
			try {
				final JavaStacktraceHyperLink hyperlink= new JavaStacktraceHyperLink(string.substring(index + 3));
				return hyperlink.resolveType();
			} catch (CoreException e) {
				WorkItemJavaIDEUIPlugin.getDefault().log(Messages.JavaElementHyperlinkHandler_EXCEPTION_RESOLVING_JAVAELEMENT, e);
			}
		}
		return null;
	}

	/**
	 * Resolves the specified uri.
	 * 
	 * @param uri the uri to resolve, which must not be <code>null</code>
	 * @return the corresponding java element, or <code>null</code> if unable to
	 * resolve the uri
	 */
	private static IJavaElement resolveQualified(final String uri) {
		Assert.isNotNull(uri);
		String string= uri.toString();
		int index= string.indexOf("://"); //$NON-NLS-1$
		if (index >= 0) {
			string= string.substring(index + 3);
			index= string.indexOf('#');
			if (index >= 0) {
				final String typeName= string.substring(0, index);
				try {
					final JavaStacktraceHyperLink hyperlink= new JavaStacktraceHyperLink(typeName);
					final IType type= hyperlink.resolveType(typeName);
					string= string.substring(index + 1);
					final StringTokenizer tokenizer= new StringTokenizer(string, ".", false); //$NON-NLS-1$
					if (tokenizer.hasMoreTokens()) {
						final String methodName= tokenizer.nextToken();
						final List<String> argumentTypes= new ArrayList<String>(8);
						while (tokenizer.hasMoreTokens())
							argumentTypes.add(tokenizer.nextToken());
						if (type != null) {
							final String[] signatures= new String[argumentTypes.size()];
							for (int offset= 0; offset < signatures.length; offset++)
								signatures[offset]= Signature.createTypeSignature(argumentTypes.get(offset), false);
							final IMethod test= type.getMethod(methodName, signatures);
							final IMethod[] methods= type.getMethods();
							for (final IMethod method : methods) {
								if (test.isSimilar(method))
									return method;
							}
							return null;
						}
					}
					return type;
				} catch (CoreException e) {
					WorkItemJavaIDEUIPlugin.getDefault().log(Messages.JavaElementHyperlinkHandler_EXCEPTION_RESOLVING_JAVAELEMENT, e);
				}
			}
		}
		return null;
	}

	/**
	 * Creates a new java element hyperlink handler.
	 * <p>
	 * Note: this class must not be instantiated. It is automatically instantiated
	 * by the extension registry. Use {@link #getInstance()} instead.
	 * </p>
	 */
	public JavaElementHyperlinkHandler() {
	// Do nothing
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public URIReference createHyperlink(final Object object, final IProgressMonitor monitor) {
		if (object instanceof IJavaElement) {
			final IJavaElement element= (IJavaElement) object;
			final String label= JavaElementLabels.getElementLabel(element, JavaElementLabels.ALL_FULLY_QUALIFIED);
			return new URIReference(label, label, IJavaElement.class.getName(), createURI(element));
		}
		return super.createHyperlink(object, monitor);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean handles(final URI uri) {
		Assert.isNotNull(uri);
		final String scheme= uri.getScheme();
		return JAVA_LINE_SCHEME.equals(scheme) || JAVA_HANDLE_SCHEME.equals(scheme) || JAVA_QUALIFIED_SCHEME.equals(scheme);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean links(final Object object) {
		if (object instanceof IJavaElement)
			return true;
		return super.links(object);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	@SuppressWarnings("restriction")
	public void open(final URI uri, final ContextProvider context, final IProgressMonitor monitor) {
		Assert.isNotNull(uri);
		final String scheme= uri.getScheme();
		final String string= uri.toString();
		if (JAVA_HANDLE_SCHEME.equals(scheme)) {
			final IJavaElement element= resolveHandle(string);
			if (element != null) {
				Display.getDefault().syncExec(new Runnable() {

					public void run() {
						try {
							openElement(element);
						} catch (PartInitException e) {
							WorkItemJavaIDEUIPlugin.getDefault().log(Messages.JavaElementHyperlinkHandler_EXCEPTION_OPENING_JAVAELEMENT, e);
						} catch (JavaModelException e) {
							WorkItemJavaIDEUIPlugin.getDefault().log(Messages.JavaElementHyperlinkHandler_EXCEPTION_OPENING_JAVAELEMENT, e);
						}
					}
				});
			}
		} else if (JAVA_LINE_SCHEME.equals(scheme)) {
			String url= null;
			try {
				url= URLDecoder.decode(string, "UTF-8"); //$NON-NLS-1$
			} catch (UnsupportedEncodingException e1) {
				// UTF-8 is assumed to be supported
				Assert.isTrue(false);
			}
			final int index= url.indexOf("://"); //$NON-NLS-1$
			final String stri= url;
			if (index >= 0) {
				Display.getDefault().syncExec(new Runnable() {

					public void run() {
						final JavaStacktraceHyperLink hyperlink= new JavaStacktraceHyperLink(stri.substring(index + 3));
						hyperlink.linkActivated();
					}
				});
			}
		} else if (JAVA_QUALIFIED_SCHEME.equals(scheme)) {
			final IJavaElement element= resolveQualified(string);
			if (element != null) {
				Display.getDefault().syncExec(new Runnable() {

					public void run() {
						try {
							openElement(element);
						} catch (PartInitException e) {
							WorkItemJavaIDEUIPlugin.getDefault().log(Messages.JavaElementHyperlinkHandler_EXCEPTION_OPENING_JAVAELEMENT, e);
						} catch (JavaModelException e) {
							WorkItemJavaIDEUIPlugin.getDefault().log(Messages.JavaElementHyperlinkHandler_EXCEPTION_OPENING_JAVAELEMENT, e);
						}
					}
				});
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Object resolve(final URI uri, final ContextProvider provider, final IProgressMonitor monitor) {
		return resolveJavaElement(uri);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean resolves(final URI uri) {
		Assert.isNotNull(uri);
		final String scheme= uri.getScheme();
		return JAVA_LINE_SCHEME.equals(scheme) || JAVA_HANDLE_SCHEME.equals(scheme) || JAVA_QUALIFIED_SCHEME.equals(scheme);

	}
}