/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.workitem.java.ide.ui.internal.editor.comments;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.search.IJavaSearchConstants;
import org.eclipse.jdt.core.search.SearchEngine;
import org.eclipse.jdt.core.search.TypeNameMatch;
import org.eclipse.jdt.core.search.TypeNameMatchRequestor;
import org.eclipse.jdt.internal.debug.ui.console.JavaStackTraceConsole;
import org.eclipse.jdt.internal.debug.ui.console.JavaStackTraceHyperlink;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleManager;
import org.eclipse.ui.console.TextConsole;

/**
 * Java implementation of a stacktrace hyperlink. TODO: request API for
 * {@link JavaStackTraceHyperlink}
 */
@SuppressWarnings("restriction")
public final class JavaStacktraceHyperLink extends JavaStackTraceHyperlink {

	/**
	 * Creates or returns a stacktrace console.
	 * 
	 * @return the stacktrace console
	 */
	private static TextConsole getStacktraceConsole() {
		final IConsoleManager manager= ConsolePlugin.getDefault().getConsoleManager();
		final IConsole[] consoles= manager.getConsoles();
		if (consoles != null) {
			for (final IConsole console : consoles) {
				if (console instanceof TextConsole && "javaStackTraceConsole".equals(console.getType())) //$NON-NLS-1$
					return (TextConsole) console;
			}
		}
		
		final TextConsole console[]= new TextConsole[1];		
		Display.getDefault().syncExec(new Runnable() { //See Defect 108162
			public void run() {
				console[0]= new JavaStackTraceConsole();
				manager.addConsoles(new IConsole[] { console[0] });
			}
		});
		
		return console[0];
	}

	/** The stacktrace line */
	private final String fLine;

	/**
	 * Creates a new java stacktrace hyperlink.
	 * 
	 * @param line the stacktrace line
	 */
	public JavaStacktraceHyperLink(String line) {
		super(getStacktraceConsole());
		Assert.isNotNull(line);
		// work around defect 50585
		int i= line.indexOf(':');
		if (i == -1) {
			int j= line.indexOf(')');
			if (j != -1)
				line= line.substring(0, j) + ":-1" + line.substring(j); //$NON-NLS-1$
		}
		fLine= line;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected String getLinkText() throws CoreException {
		return fLine;
	}

	/**
	 * Returns the fully qualified type name.
	 * 
	 * @return the fully qualified type name, which is never <code>null</code>
	 * @throws CoreException
	 */
	public String getTypeName() throws CoreException {
		return getTypeName(fLine);
	}

	/**
	 * Resolves the type of this hyperlink.
	 * 
	 * @return the resolved type, or <code>null</code> if unable to resolve
	 * @throws CoreException if an error occurs while resolving the type
	 */
	public IType resolveType() throws CoreException {
		return resolveType(getTypeName());
	}

	/**
	 * Resolves the type of this hyperlink.
	 * 
	 * @param typeName the type name, which must not be <code>null</code>
	 * @return the resolved type, or <code>null</code> if unable to resolve
	 * @throws CoreException if an error occurs while resolving the type
	 */
	public IType resolveType(final String typeName) throws CoreException {
		return findTypeInWorkspace(typeName);
	}
	
	/**
	 * Searches for and returns a type with the given name in the workspace,
	 * or <code>null</code> if none.
	 * 
	 * @param typeName fully qualified type name
	 * @return type or <code>null</code>
	 * @throws JavaModelException
	 */
	private static IType findTypeInWorkspace(String typeName) throws CoreException {
		int dot= typeName.lastIndexOf('.');
		char[][] qualifications;
		String simpleName;
		if (dot != -1) {
			qualifications= new char[][] { typeName.substring(0, dot).toCharArray() };
			simpleName= typeName.substring(dot + 1);
		} else {
			qualifications= null;
			simpleName= typeName;
		}
		char[][] typeNames= new char[][] { simpleName.toCharArray() };
		
		class ResultException extends RuntimeException {
			private static final long serialVersionUID= 1L;
			private final IType fType;
			public ResultException(IType type) {
				fType= type;
			}
		}
		TypeNameMatchRequestor requestor= new TypeNameMatchRequestor() {
			public void acceptTypeNameMatch(TypeNameMatch match) {
				throw new ResultException(match.getType());
			}
		};
		try {
			new SearchEngine().searchAllTypeNames(qualifications, typeNames, SearchEngine.createWorkspaceScope(), requestor, IJavaSearchConstants.WAIT_UNTIL_READY_TO_SEARCH, null);
		} catch (ResultException e) {
			return e.fType;
		}
		return null;
	}
}