/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.java.ide.ui.internal.editor.comments.java;

import com.ibm.team.foundation.rcp.core.text.ReferencePosition;
import com.ibm.team.workitem.ide.ui.internal.editor.comments.ILineBasedSubstitutionScanner;
import com.ibm.team.workitem.java.ide.ui.internal.editor.comments.JavaElementHyperlinkHandler;

import java.net.URI;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class JavaStackTraceScanner extends ILineBasedSubstitutionScanner {

	protected static final String JAVA_IDENTIFIER= "[\\w\\p{Sc}]+"; //$NON-NLS-1$
	protected static final String JAVA_CLASS_NAME= "((" + JAVA_IDENTIFIER + "\\.)*" + JAVA_IDENTIFIER + ")"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	private static final Pattern STACK_TRACE_PATTERN= Pattern.compile("^\\s*at\\s+" + JAVA_CLASS_NAME + "\\." + JAVA_IDENTIFIER + "\\((" + JAVA_IDENTIFIER + "\\.java:(\\S*))\\)"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$

	public boolean substitute(String inputLine, StringBuilder outputBuffer, int outputOffset) {
		Matcher matcher= createMatcher(outputBuffer.substring(outputOffset, outputBuffer.length()));
		if (!matcher.find())
			return false;
		
		URI uri= createURI(matcher);
		if (uri == null)
			return false;
		
		int linkGroup= getLinkGroup(matcher);
		if (linkGroup == -1)
			return false;
		
		int openOffset= outputOffset + matcher.start(linkGroup);
		String openTag= "<a href=\"" + uri.toString() + "\">"; //$NON-NLS-1$ //$NON-NLS-2$
		outputBuffer.insert(openOffset, openTag);
		int closeOffset= openOffset + openTag.length() + matcher.end(linkGroup) - matcher.start(linkGroup);
		outputBuffer.insert(closeOffset, "</a>"); //$NON-NLS-1$
		return true;
	}
	
	public ReferencePosition[] scan(String line) {
		Matcher matcher= createMatcher(line);
		if (!matcher.find())
			return new ReferencePosition[0];
		
		URI uri= createURI(matcher);
		if (uri == null)
			return new ReferencePosition[0];
		
		int linkGroup= getLinkGroup(matcher);
		if (linkGroup == -1)
			return new ReferencePosition[0];
		
		int offset= matcher.start(linkGroup);
		int length= matcher.end(linkGroup) - matcher.start(linkGroup);
		return new ReferencePosition[] { new ReferencePosition(offset, length, uri, true) };
	}

	protected Matcher createMatcher(String line) {
		return STACK_TRACE_PATTERN.matcher(line);
	}

	protected int getLinkGroup(Matcher matcher) {
		return 3;
	}

	protected int getTypeGroup(Matcher matcher) {
		return 1;
	}

	protected int getLineGroup(Matcher matcher) {
		return 4;
	}
	
	protected URI createURI(Matcher matcher) {
		int line= getLineNumber(matcher);
		String typeName= getTypeName(matcher);
		if (typeName != null)
			return JavaElementHyperlinkHandler.createURI(typeName, line);
		return null;
	}

	/**
	 * Returns the fully qualified name of the type to open
	 * 
	 * @return fully qualified type name
	 */
	protected final String getTypeName(Matcher matcher) {
		int typeGroup= getTypeGroup(matcher);
		if (typeGroup == -1)
			return null;
		
		String typeName= matcher.group(typeGroup);
		int innerClassIndex= typeName.indexOf('$');
		if (innerClassIndex != -1)
			typeName= typeName.substring(0, innerClassIndex);
		return typeName;
	}

	/**
	 * Returns the line number associated with the stack trace or -1 if
	 * none.
	 */
	protected final int getLineNumber(Matcher matcher) {
		int lineGroup= getLineGroup(matcher);
		if (lineGroup == -1)
			return -1;
		
		try {
			return Integer.parseInt(matcher.group(lineGroup));
		} catch (NumberFormatException e) {
		}
		return -1;
	}
}
