/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.workitem.java.ide.ui.internal.integration;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.ISafeRunnable;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.ui.text.java.IInvocationContext;
import org.eclipse.jdt.ui.text.java.IJavaCompletionProposal;
import org.eclipse.jdt.ui.text.java.IProblemLocation;
import org.eclipse.jdt.ui.text.java.IQuickFixProcessor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.resource.LocalResourceManager;
import org.eclipse.jface.resource.ResourceManager;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.util.SafeRunnable;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import com.ibm.team.foundation.common.URIReference;
import com.ibm.team.foundation.common.text.XMLString;
import com.ibm.team.foundation.ide.ui.internal.styledtextviewer.contentassist.AdditionalInformationInput;
import com.ibm.team.foundation.ide.ui.internal.styledtextviewer.contentassist.CompletionProposal;
import com.ibm.team.foundation.rcp.core.hyperlinks.Hyperlinks;
import com.ibm.team.links.common.IReference;
import com.ibm.team.links.common.factory.IReferenceFactory;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.workitem.client.WorkItemWorkingCopy;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.WorkItemEndPoints;
import com.ibm.team.workitem.ide.ui.internal.WorkItemIDEUIPlugin;
import com.ibm.team.workitem.ide.ui.internal.integration.CreateWorkItemAction;
import com.ibm.team.workitem.java.ide.ui.internal.WorkItemJavaIDEUIPlugin;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;

/**
 * An implementation of {@link IQuickFixProcessor} to convert a Task in Java
 * Source into a Work Item.
 */
public class TaskQuickFixProcessor implements IQuickFixProcessor {

	@SuppressWarnings("restriction")
	private static class ConvertTaskToWorkItemProposal extends CompletionProposal implements IJavaCompletionProposal {
		private final String fTaskText;
		private final IWorkbenchPart fPart;
		private final IResource fResource;

		public ConvertTaskToWorkItemProposal(IResource resource, String displayString, String taskText, Image image, IWorkbenchPart activePart) {
			super(displayString, image);
			fResource= resource;
			fTaskText= taskText;
			fPart= activePart;
		}

		@Override
		public void apply(IDocument document, char trigger, int offset) {
			CreateWorkItemAction action= new CreateWorkItemAction(false) {

				@Override
				protected void updateEnablement(IAction action, ISelection selection) {}

				@Override
				protected void initializeWorkItem(WorkItemWorkingCopy workingCopy, ISelection selection, IProgressMonitor monitor) throws TeamRepositoryException {
					String summary= extractSummary();
					if (summary == null)
						return;

					IWorkItem workItem= workingCopy.getWorkItem();
					workItem.setHTMLSummary(XMLString.createFromPlainText(summary.trim()));

					String content= extractContent(selection);
					if (content != null)
						workItem.setHTMLDescription(XMLString.createFromPlainText(content.trim()));

					addLink(workingCopy, fResource);
				}

				private String extractSummary() {
					return fTaskText;
				}

				@Override
				protected String extractContent(ISelection selection) {
					if (fResource != null)
						return NLS.bind(Messages.TaskQuickFixProcessor_RESOURCE, fResource.getFullPath().toString());

					return null;
				}

				private void addLink(final WorkItemWorkingCopy workingCopy, final IResource resource) {
					if (resource != null) {
						SafeRunnable.run(new ISafeRunnable() {
							public void run() throws Exception {
								URI uri= resource.getLocationURI();
								URIReference uriReference= Hyperlinks.create(resource, null);
								if (uriReference != null) {
									IReference reference= IReferenceFactory.INSTANCE.createReferenceFromURI(uri != null ? uri : uriReference.getURI(), uriReference.getName());
									workingCopy.getReferences().add(WorkItemEndPoints.RELATED_ARTIFACT, reference);
								}
							}

							public void handleException(Throwable exception) {
								WorkItemIDEUIPlugin.getDefault().log(exception.getMessage(), exception);
							}
						});
					}
				}
			};
			action.setActivePart(null, fPart);
			action.run(null);
		}

		@Override
		protected AdditionalInformationInput getAdditionaInformationInput(IProgressMonitor monitor) {
			return null;
		}

		@Override
		public boolean isValidFor(IDocument document, int offset) {
			return true;
		}

		public int getRelevance() {
			return 0;
		}
	}

	private ResourceManager fResources= new LocalResourceManager(JFaceResources.getResources());
	private ImageDescriptor fImage= WorkItemJavaIDEUIPlugin.imageDescriptorFromPlugin(WorkItemJavaIDEUIPlugin.PLUGIN_ID, "icons/elcl16/crtworkitem.gif"); //$NON-NLS-1$

	/*
	 * @see org.eclipse.jdt.ui.text.java.IQuickFixProcessor#hasCorrections(org.eclipse.jdt.core.ICompilationUnit, int)
	 */
	public boolean hasCorrections(ICompilationUnit unit, int problemId) {
		return true; //Handled by extension already
	}

	/*
	 * @see org.eclipse.jdt.ui.text.java.IQuickFixProcessor#getCorrections(org.eclipse.jdt.ui.text.java.IInvocationContext, org.eclipse.jdt.ui.text.java.IProblemLocation[])
	 */
	public IJavaCompletionProposal[] getCorrections(IInvocationContext context, IProblemLocation[] locations) throws CoreException {
		List<IJavaCompletionProposal> proposals= new ArrayList<IJavaCompletionProposal>();

		IWorkbenchWindow window= PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		if (window != null) {
			IWorkbenchPage activePage= window.getActivePage();
			if (activePage != null) {
				IWorkbenchPart activePart= activePage.getActivePart();
				if (activePart != null) {
					for (IProblemLocation location : locations) {
						ICompilationUnit cu= context.getCompilationUnit();
						String taskText= cu.getBuffer().getText(location.getOffset(), location.getLength());

						ConvertTaskToWorkItemProposal proposal= new ConvertTaskToWorkItemProposal(cu.getResource(), Messages.TaskQuickFixProcessor_NEW_WORK_ITEM, taskText, fResources.createImage(fImage), activePart);
						proposals.add(proposal);
					}
				}
			}
		}

		return proposals.toArray(new IJavaCompletionProposal[proposals.size()]);
	}
}